import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.concurrent.CyclicBarrier;

import javax.swing.JPanel;

public class GamePanel extends JPanel implements Runnable, MouseListener, KeyListener {
	
	private static final int PWIDTH = 500; // size of panel
	private static final int PHEIGHT = 500;

	private Thread animator; // the thread that performs the animation
	private static final int SLEEP_MILLIS = 10; // thread-sleep per iteration
	long lastUpdate = System.currentTimeMillis();
	private boolean running = false;
	private int movementMode = 0;

	Font font = new Font("SansSerif", Font.BOLD, 12); // set up message font to display text
	FontMetrics metrics = this.getFontMetrics(font);

	double baseLineY = PHEIGHT / 2; // object moves horizontally along this baseline
	double baseLineX = PWIDTH / 2; 
	double angle = 0;				// angle in radians
	double xPos = 0, yPos = 0;		// position of object (ball)
	
	// current position of the ball
	int ballSize = 40;

	public GamePanel() 
	{
		setBackground(Color.white);
		setPreferredSize(new Dimension(PWIDTH, PHEIGHT));
		setFocusable(true);
		addKeyListener(this);
        addMouseListener(this);
		requestFocus();
	}

	public void addNotify()	
	{
		super.addNotify(); 
		startGame(); 
	}

	private void startGame()
	{
		if (animator == null || !running) {
			animator = new Thread(this);
			animator.start();
		}
	}

	public void run()
	{
		running = true;
		while (running) {
			gameUpdate();
			paintScreen(); // draw on-screen
			try {
				Thread.sleep(SLEEP_MILLIS);
			} catch (InterruptedException e) { }
		}
		System.exit(0); 
	}

	/**
	 * Moves xPos and yPos along a sinus-curve. 
	 */
	private void gameUpdate() 
	{
		long diffTime = System.currentTimeMillis() - lastUpdate;
		float diffTimeSec = (float)diffTime/1000.0f;

		switch(movementMode % 3) {
		case 0:		// circular movement around the center
			calculateCyclic(diffTimeSec);
			break;
		case 1: 	// movement along a sinus-curve
			calculateSinus(diffTimeSec);
			break;
		case 2:		// mix both movement styles (stays within circle)
			calculateCyclic(diffTimeSec);
			calculateSinus(diffTimeSec);
			break;
		}
		
		lastUpdate = System.currentTimeMillis();
	}

	/**
	 * Moves xPos and yPos along a sinus-curve.
	 * @param diffTimeSec	time since last update in seconds
	 */
	private void calculateSinus(float diffTimeSec) {
		double amplitude = 100; 	// amplitude in pixel (size of sinus-curve)
		double frequency = 6; 		// number of oscillations (per screen-width)
		double xSpeed = 50; 		// horizontal speed in pixels per second
		if (xPos > PWIDTH) xPos = 0;
		xPos = xPos + (double)diffTimeSec * xSpeed;
		double x = ((2*Math.PI/PWIDTH)) * xPos * frequency;	
		double y = Math.sin(x);
		yPos = y*amplitude + baseLineY;
	}

	/**
	 * Moves xPos and yPos around the center of the screen with a given radius.
	 * @param diffTimeSec	time since last update in seconds
	 */
	private void calculateCyclic(float diffTimeSec) {
		double radius = 150.0;
		double angleDisplacement = (Math.PI/2) * diffTimeSec; // rotate PI/2 per second
		angle = angle + angleDisplacement;
		if (angle > (Math.PI*2)) angle = 0;
		xPos = baseLineX + (radius * Math.cos(angle));
		yPos = baseLineY + (radius * Math.sin(angle));
	}

	private void paintScreen() {
		Graphics g = this.getGraphics();

		// clear the background (white) and set the font.
		g.setColor(Color.white);
		g.fillRect(0, 0, PWIDTH, PHEIGHT);
		g.setColor(Color.blue);
		g.setFont(font);

		// draw baselines
		g.setColor(Color.black);
		g.drawLine(0, PHEIGHT/2, PWIDTH, PHEIGHT/2);
		g.drawLine(PWIDTH/2, 0, PWIDTH/2, PHEIGHT);
		
		// draw ball
		g.setColor(Color.blue);
		g.fillOval((int)xPos-ballSize/2, (int)yPos-ballSize/2, ballSize, ballSize);

		
	} 
	
	// -------------------------------------------------------------------------------
	
	// Mouse Events
	public void mouseClicked(MouseEvent e) { }
	public void mouseReleased(MouseEvent e) { }
    public void mouseEntered(MouseEvent e) { }
    public void mousePressed(MouseEvent e) { }
    public void mouseExited(MouseEvent e) { }
	
	// Keyboard Events
    public void keyPressed(KeyEvent e) 
    {
    	if (e.getKeyCode() == KeyEvent.VK_SPACE) movementMode++;
    	if (e.getKeyCode() == KeyEvent.VK_ESCAPE) running = false;
    }
    public void keyTyped(KeyEvent e) { }
    public void keyReleased(KeyEvent e) { }

	// called from container (on window events)
	public void stopGame() 
	{
		running = false;
	}
    
}


